/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.model;

public final class DataFetcherJobBuilder {
    private String idwsMessageId;
    private JobType jobType;
    private String citizenCpr;

    private DataFetcherJobBuilder() {
    }

    public static DataFetcherJobBuilder aDataFetcherJob() {
        return new DataFetcherJobBuilder();
    }

    public DataFetcherJobBuilder withIdwsMessageId(String idwsMessageId) {
        this.idwsMessageId = idwsMessageId;
        return this;
    }

    public DataFetcherJobBuilder withJobType(JobType jobType) {
        this.jobType = jobType;
        return this;
    }

    public DataFetcherJobBuilder withCitizenCpr(String citizenCpr) {
        this.citizenCpr = citizenCpr;
        return this;
    }

    public DataFetcherJob build() {
        DataFetcherJob dataFetcherJob = new DataFetcherJob();
        dataFetcherJob.setIdwsMessageId(idwsMessageId);
        dataFetcherJob.setJobType(jobType);
        dataFetcherJob.setCitizenCpr(citizenCpr);
        return dataFetcherJob;
    }
}
